% Estimate CSI weights -- alternatives

clear all;
small = 1.0e-10;
big = 1.0e+6;                  
rng(63761);

p1 = path;
path(p1,'../m_utilities');

 % -- File Directories  
  outdir = 'out/';
  figdir = 'fig/';
  matdir = 'mat/';
  
  % -- Read in Data --- 
  load_data = 0;  % 1 if reloading data from Excel, etc 
  mtoq_agg = 0;   % Temporal aggregation indicator of monthly to quarterly data
  pc_comp_data_calendar_m_and_q_prices;
  pc_comp_data_calendar_m_and_q_real;
  % Data Series Used
  dp_agg = dp_agg_q;
  dp_agg_xfe = dp_agg_xfe_q;
  dp_agg_xe = dp_agg_xe_q;
  dp_aggs = [dp_agg dp_agg_xfe dp_agg_xe];
  namevec_aggs = {'dp_agg';'dp_agg_xfe';'dp_agg_xe'};
  dp_disagg = dp_disagg_q;
  calvec_p = calvec_qp;
  calvec_r = calvec_qr;
  dnobs_p = dnobs_qp;
  calds_p = calds_qp;
  dp_disagg_zero = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]';   % Coefficients set to zero
  'Coefficients set to zero:'
  labelvec_disagg(dp_disagg_zero==1)
  dp_non_zero = 1 - dp_disagg_zero;
  n_p = size(dp_disagg,2);
  dp_disagg_yoy = dif(ma(dp_disagg,4),4);
  n_p = size(dp_disagg,2);
  
  % BP Parameters
  n = 40;
  nar = 6;
  tcode = 1;
  seflag = 0;
  lpdc = 32;
  updc = 6;
  
  % Construct BP disagg inflation
  dp_disagg_bp = NaN(size(dp_disagg));
  for i = 1:size(dp_disagg,2);
      [xbp tmp] = bpassar(dp_disagg(:,i),updc,lpdc,n,nar,tcode,seflag);
      dp_disagg_bp(:,i) = xbp;
  end;
 
  % Load CAI
  str = [matdir 'cai'];load(str);
  
  % Iteration Parameters
  nit = 1500;
  options = optimset('Display','off');
  
first_date = [1984 1];
last_date = [2019 1];
ismpl_p = smpl(calvec_p,first_date,last_date,4);
ismpl_r = smpl(calvec_r,first_date,last_date,4);
calvec_est = calvec_p(ismpl_p==1);

%%%%% % Alternative 1: Zeros on poorly measured sectors %%%%%% 
  dp_disagg_zero = [0 0 1 0 0 1 0 0 0 0 0 0 0 0 1 0 1]';   % Coefficients set to zero
  'Coefficients set to zero:'
  labelvec_disagg(dp_disagg_zero==1)
  dp_non_zero = 1 - dp_disagg_zero;
  
  % Step 1:  Regress ybp_pc onto inflation measures .. compute R2
  y = cai(ismpl_r==1);
  x = dp_disagg_yoy(ismpl_p==1,dp_non_zero'==1);
  x = x - mean(x);
  fdata.z = x;
  fdata.y = y;
  theta_max = zeros(size(x,2)-1,1);
  fmin = 0;
  fmin_save = fmin;
  theta_save = theta_max;   
  tic;
  for i = 1:nit;
    if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
    else;
      theta = 0.5*randn(size(x,2)-1,1);
    end;
    theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
    tmp = myfun4(theta,fdata);
    if tmp < fmin;
      theta_max = theta;
      fmin = tmp;
      theta_save = [theta_max theta_save];
      fmin_save = [fmin;fmin_save];
    end;
  end;
  b_logit = logit_wght(theta_save(:,1));
  w_logit = zeros(n_p,1);
  w_logit(dp_non_zero==1) = b_logit;

  % Save CSI weights
  str_tmp = [matdir 'w_logit_poorlymeasured_excluded'];
  save(str_tmp,'w_logit');
  
%%%%% %  Alternative 2: Use BP Inflation %%%%%%
   dp_disagg_zero = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]';   % Coefficients set to zero
  'Coefficients set to zero:'
  labelvec_disagg(dp_disagg_zero==1)
  dp_non_zero = 1 - dp_disagg_zero;
 
  % Step 1:  Regress ybp_pc onto inflation measures .. compute R2
  y = cai(ismpl_r==1);
  x = dp_disagg_bp(ismpl_p==1,dp_non_zero'==1);
  x = x - mean(x);
  fdata.z = x;
  fdata.y = y;
  theta_max = zeros(size(x,2)-1,1);
  fmin = 0;
  fmin_save = fmin;
  theta_save = theta_max;   
  tic;
  for i = 1:nit;
    if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
    else;
      theta = 0.5*randn(size(x,2)-1,1);
    end;
    theta = fminsearch(@(x) myfun4(x,fdata),theta,options); 
    tmp = myfun4(theta,fdata);
    if tmp < fmin;
      theta_max = theta;
      fmin = tmp;
      theta_save = [theta_max theta_save];
      fmin_save = [fmin;fmin_save];
    end;
  end;
  b_logit = logit_wght(theta_save(:,1));
  w_logit = zeros(n_p,1);
  w_logit(dp_non_zero==1) = b_logit;

  % Save CSI weights
  str_tmp = [matdir 'w_logit_bp_inflation'];
  save(str_tmp,'w_logit');
  
 %%%%% % Alternative 3: Impose largest coefficient = 0.20 %%%%%% 
  dp_disagg_zero = [0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0]';   % Coefficients set to zero
  'Coefficients set to zero:'
  labelvec_disagg(dp_disagg_zero==1)
  dp_non_zero = 1 - dp_disagg_zero;
  
  % Step 1:  Regress ybp_pc onto inflation measures .. compute R2
  y = cai(ismpl_r==1);
  x = dp_disagg_yoy(ismpl_p==1,dp_non_zero'==1);
  x = x - mean(x);
  fdata.z = x;
  fdata.y = y;
  theta_max = zeros(size(x,2)-1,1);
  fmin = 0;
  fmin_save = fmin;
  theta_save = theta_max;   
  tic;
  for i = 1:nit;
    if floor(i/2) == ceil(i/2);
      theta = theta + .02*randn(size(x,2)-1,1);
    else;
      theta = 0.5*randn(size(x,2)-1,1);
    end;
    theta = fminsearch(@(x) myfun5(x,fdata),theta,options); 
    tmp = myfun5(theta,fdata);
    if tmp < fmin;
      theta_max = theta;
      fmin = tmp;
      theta_save = [theta_max theta_save];
      fmin_save = [fmin;fmin_save];
    end;
  end;
  b_logit = logit_wght(theta_save(:,1));
  w_logit = zeros(n_p,1);
  w_logit(dp_non_zero==1) = b_logit;

  % Save CSI weights
  str_tmp = [matdir 'w_logit_20_max'];
  save(str_tmp,'w_logit');
  
  path(p1);


  